#include "stdafx.h"

#include <fstream>
#include <algorithm>

#include "Util.h"
#include "Logger.h"
#include "AddressInfo.h"

////////////////////////////////////////////////////////////////////////
// [AhX̏
////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------
// RXgN^
//----------------------------------------------------------------------

MailAddressInfo::MailAddressInfo() : 
	m_addrType(AddressEmail)
{
}

//----------------------------------------------------------------------
// RXgN^
//----------------------------------------------------------------------

MailAddressInfo::MailAddressInfo(const char* email, const char* note) :
	m_email(email),
	m_note(note)
{
	const char* pDomain = strchr(email, '@');
	if (pDomain) {
		m_addrType = AddressEmail;
	} else {
		m_addrType = AddressDomain;
	}
}

//----------------------------------------------------------------------
// AhX̎
//----------------------------------------------------------------------

MailAddressInfo::AddressType MailAddressInfo::GetAddressType()
{
	return m_addrType;
}

//----------------------------------------------------------------------
// [AhX
//----------------------------------------------------------------------

std::string& MailAddressInfo::GetMailAddress()
{
	return m_email;
}

//----------------------------------------------------------------------
// l
//----------------------------------------------------------------------

std::string& MailAddressInfo::GetNote()
{
	return m_note;
}

////////////////////////////////////////////////////////////////////////
// zCgXg
////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------
// RXgN^
//----------------------------------------------------------------------

WhiteList::WhiteList() :
	m_pcsWhiteList(0),
	m_sortKey(SortAddrType),
	m_bAscending(true)
{
}

//----------------------------------------------------------------------
// [AhXуhC݂邩𓾂
//----------------------------------------------------------------------

bool WhiteList::IsExist(const char* email)
{
	CSLock cslock(m_pcsWhiteList);

	std::map<std::string, MailAddressInfo>::iterator it = m_mailList.find(email);
	if (it != m_mailList.end()) {
		return true;
	}

	return false;
}

//----------------------------------------------------------------------
// [AhXуhC݂邩𓾂(zCgXg`FbNp)
//----------------------------------------------------------------------

bool WhiteList::IsExistEmailOrDomain(const char* email)
{
	CSLock cslock(m_pcsWhiteList);

	std::map<std::string, MailAddressInfo>::iterator it;

	it = m_mailList.find(email);
	if (it != m_mailList.end()) {
		return true;
	}

	std::string domain = Util::GetAtNext(email);
	if (domain.length() == 0) {
		return false;
	}

	it = m_mailList.find(domain);
	if (it != m_mailList.end()) {
		return true;
	}

	return false;
}

//----------------------------------------------------------------------
// [AhX݂ȂA͔l󂩂𓾂
//----------------------------------------------------------------------

bool WhiteList::IsNotExistOrNoteIsEmpty(const char* email, bool* pbNoteIsEmpty)
{
	CSLock cslock(m_pcsWhiteList);

	if (pbNoteIsEmpty) {
		*pbNoteIsEmpty = false;
	}

	std::map<std::string, MailAddressInfo>::iterator it = m_mailList.find(email);
	if (it != m_mailList.end()) {
		if (it->second.GetNote().length() == 0) {
			if (pbNoteIsEmpty) {
				*pbNoteIsEmpty = true;
			}
			return true;
		}

		return false;
	}

	return true;
}

//----------------------------------------------------------------------
// [AhX̏𓾂
//----------------------------------------------------------------------

std::map<std::string, MailAddressInfo>& WhiteList::GetMailAddressList()
{
	CSLock cslock(m_pcsWhiteList);

	return m_mailList;
}

//----------------------------------------------------------------------
// [AhX̌𓾂
//----------------------------------------------------------------------

int WhiteList::Count()
{
	CSLock cslock(m_pcsWhiteList);

	return m_mailList.size();
}

//----------------------------------------------------------------------
// [AhX̏ǉ
//----------------------------------------------------------------------

void WhiteList::Add(const char* email, const char* note)
{
	CSLock cslock(m_pcsWhiteList);

	MailAddressInfo minfo(email, note);
	m_mailList[email] = minfo;

	// \[gp
	std::vector<std::string>::iterator it = std::find(m_sortedEMailAddr.begin(), m_sortedEMailAddr.end(), email);
	if (it == m_sortedEMailAddr.end()) {
		m_sortedEMailAddr.push_back(email);
	}
}

//----------------------------------------------------------------------
// 폜
//----------------------------------------------------------------------

void WhiteList::Delete(const char* email)
{
#define __FUNCNAME__ "WhiteList::Delete()"

	CSLock cslock(m_pcsWhiteList);

	m_mailList.erase(email);

	// \[gp
	std::vector<std::string>::iterator it = std::find(m_sortedEMailAddr.begin(), m_sortedEMailAddr.end(), email);
	if (it != m_sortedEMailAddr.end()) {
		m_sortedEMailAddr.erase(it);
	}

#undef __FUNCNAME__
}

//----------------------------------------------------------------------
// XV
//----------------------------------------------------------------------

void WhiteList::Update(const char* email, const char* note)
{
	// std::map Ȃ̂ŁAgAꉞ֐𕪂Ă
	Add(email, note);
}

//----------------------------------------------------------------------
// ǉ܂͍XV
//----------------------------------------------------------------------

void WhiteList::AddOrUpdate(const char* email, const char* note)
{
	// std::map Ȃ̂ŁAgAꉞ֐𕪂Ă
	Add(email, note);
}

//----------------------------------------------------------------------
// SNA
//----------------------------------------------------------------------

void WhiteList::ClearAll()
{
	CSLock cslock(m_pcsWhiteList);

	m_mailList.clear();

	// \[gp
	m_sortedEMailAddr.clear();
}

//----------------------------------------------------------------------
// C|[g
//----------------------------------------------------------------------

void WhiteList::Import(const char* filepath)
{
	CSLock cslock(m_pcsWhiteList);

	const char* separator = "\t";

	m_mailList.clear();
	m_sortedEMailAddr.clear();	// \[gp

	std::ifstream mailListfile;
	mailListfile.open(filepath, std::ios::in);
	if (mailListfile) {
		std::string line;
		while (std::getline(mailListfile, line)) {

			std::string maildata = Util::Trim(line);

			std::string email = "";
			std::string note = "";

			int pos = maildata.find_first_of(separator);
			if (pos == std::string::npos) {

				email = Util::Trim(maildata);
				email = email.substr(0, min(NUM_CHAR_MAIL_ADDRESS_MAX, email.length()));

			} else {

				email = maildata.substr(0, pos);
				note = maildata.substr(pos + 1);

				email = Util::Trim(email);
				note = Util::Trim(note);

				email = email.substr(0, min(NUM_CHAR_MAIL_ADDRESS_MAX, email.length()));
				note = note.substr(0, min(NUM_CHAR_MAIL_NOTE_MAX, note.length()));
			}

			if (0 < email.length()) {
				Add(email.c_str(), note.c_str());
			}
		}

		mailListfile.close();
	}
}

//----------------------------------------------------------------------
// GNX|[g
//----------------------------------------------------------------------

void WhiteList::Export(const char* filepath)
{
	CSLock cslock(m_pcsWhiteList);

	std::ofstream mailListfile;
	mailListfile.open(filepath, std::ios::out | std::ios::trunc);
	if (mailListfile) {

		for(std::map<std::string, MailAddressInfo>::iterator it = m_mailList.begin(); it != m_mailList.end(); ++it) {
			mailListfile << it->second.GetMailAddress() << '\t' << it->second.GetNote() << std::endl;
		}

		mailListfile.close();
	}
}

//----------------------------------------------------------------------
// rp̃IuWFNgݒ肷
//----------------------------------------------------------------------

void WhiteList::SetLockObject(CRITICAL_SECTION* pcsWhiteList)
{
	m_pcsWhiteList = pcsWhiteList;
}

//----------------------------------------------------------------------
// \[gs
//----------------------------------------------------------------------

//
// \[gp
//
class SortPred {
public:
	// RXgN^
	SortPred(WhiteList::SortKey sortKey, bool bAscending, std::map<std::string, MailAddressInfo>& mailList) :
		m_mailList(mailList)
	{
		m_sortKey = sortKey;
		m_bAscending = bAscending;
	}

	// r֐
	bool operator()(std::string& left, std::string& right) {
		bool result = false;

		std::map<std::string, MailAddressInfo>::iterator itLeft = m_mailList.find(left);
		std::map<std::string, MailAddressInfo>::iterator itRight = m_mailList.find(right);

		// Ce[^ m_mailList.end() ̏ꍇ̃`FbN͔Ô߂̃R[hŁA݂̎gł͋NȂ(͂)B
		if (itLeft == m_mailList.end() && itRight == m_mailList.end()) {
			result = true;
		} else if (itLeft == m_mailList.end() && itRight != m_mailList.end()) {
			result = true;
		} else if (itLeft != m_mailList.end() && itRight == m_mailList.end()) {
			result = false;
		} else {

			const int sortKeyNum = 3;
			WhiteList::SortKey sortKeyArr[sortKeyNum];

			switch (m_sortKey) {
			case WhiteList::SortAddrType:
			default:
				sortKeyArr[0] = WhiteList::SortAddrType;
				sortKeyArr[1] = WhiteList::SortEmailAddr;
				sortKeyArr[2] = WhiteList::SortNote;
				break;
			case WhiteList::SortEmailAddr:
				sortKeyArr[0] = WhiteList::SortEmailAddr;
				sortKeyArr[1] = WhiteList::SortAddrType;
				sortKeyArr[2] = WhiteList::SortNote;
				break;
			case WhiteList::SortNote:
				sortKeyArr[0] = WhiteList::SortNote;
				sortKeyArr[1] = WhiteList::SortAddrType;
				sortKeyArr[2] = WhiteList::SortEmailAddr;
				break;
			}

			int i = 0;
			for ( ; i < sortKeyNum; ++i) {

				switch (sortKeyArr[i]) {
				case WhiteList::SortAddrType:
				default:
					if (itLeft->second.GetAddressType() < itRight->second.GetAddressType()) {
						result = true;
					} else if (itLeft->second.GetAddressType() == itRight->second.GetAddressType()) {
						continue;
					} else {
						result = false;
					}
					break;

				case WhiteList::SortEmailAddr:
					{
						int comprc = itLeft->first.compare(itRight->first);
						if (comprc < 0) {
							result = true;
						} else if (comprc == 0) {
							continue;	// 蓾Ȃ(͂)BÔ
						} else {
							result = false;
						}
					}
					break;

				case WhiteList::SortNote:
					{
						int comprc = itLeft->second.GetNote().compare(itRight->second.GetNote());
						if (comprc < 0) {
							result = true;
						} else if (comprc == 0) {
							continue;
						} else {
							result = false;
						}
					}
					break;
				}

				break;
			}

			if (sortKeyNum <= i) {
				result = true;
			}
		}
		
		if (!m_bAscending) {
			result = !result;
		}

		return result;
	}

private:
	// \[gL[
	WhiteList::SortKey m_sortKey;
	// ǂ
	bool m_bAscending;
	// [AhX̏
	std::map<std::string, MailAddressInfo>& m_mailList;
};

//----------------------------------------------------------------------
// \[gs
//----------------------------------------------------------------------

void WhiteList::Sort()
{
	CSLock cslock(m_pcsWhiteList);

	std::sort(m_sortedEMailAddr.begin(), m_sortedEMailAddr.end(), SortPred(m_sortKey, m_bAscending, m_mailList));
}

//----------------------------------------------------------------------
// \[gL[擾/ݒ
//----------------------------------------------------------------------

WhiteList::SortKey WhiteList::GetSortKey()
{
	CSLock cslock(m_pcsWhiteList);

	return m_sortKey;
}

void WhiteList::SetSortKey(SortKey sortKey)
{
	CSLock cslock(m_pcsWhiteList);

	m_sortKey = sortKey;
}

//----------------------------------------------------------------------
// ǂ擾/ݒ
//----------------------------------------------------------------------

bool WhiteList::IsAscending()
{
	CSLock cslock(m_pcsWhiteList);

	return m_bAscending;
}

void WhiteList::SetAscending(bool bAscending)
{
	CSLock cslock(m_pcsWhiteList);

	m_bAscending = bAscending;
}

//----------------------------------------------------------------------
// n Ԗڂ̃[AhX̏Ԃ
//----------------------------------------------------------------------

MailAddressInfo& WhiteList::GetAt(int index)
{
	CSLock cslock(m_pcsWhiteList);

	if (index < (int) m_sortedEMailAddr.size()) {
		std::map<std::string, MailAddressInfo>::iterator it = m_mailList.find(m_sortedEMailAddr[index]);
		if (it != m_mailList.end()) {
			return it->second;
		}
	}

	// ւ͂Ȃ(͂)
	return m_emptyMai;
}

//----------------------------------------------------------------------
// w̃[AhX̃CfbNXԂ
//----------------------------------------------------------------------

int WhiteList::GetIndex(const char* email)
{
	CSLock cslock(m_pcsWhiteList);

	std::vector<std::string>::iterator it = std::find(m_sortedEMailAddr.begin(), m_sortedEMailAddr.end(), email);
	if (it == m_sortedEMailAddr.end()) {
		return -1;
	}

	int index = std::distance(m_sortedEMailAddr.begin(), it);
	return index;
}

//----------------------------------------------------------------------
// n Ԗڂ̃[AhX̏폜
//----------------------------------------------------------------------

bool WhiteList::DeleteAt(int index)
{
	CSLock cslock(m_pcsWhiteList);

	if (index < (int) m_sortedEMailAddr.size()) {
		std::map<std::string, MailAddressInfo>::iterator it = m_mailList.find(m_sortedEMailAddr[index]);
		if (it != m_mailList.end()) {
			std::string email = it->first;
			Delete(email.c_str());
			return true;
		}
	}

	return false;
}

////////////////////////////////////////////////////////////////////////
// g̃[AhX̏
////////////////////////////////////////////////////////////////////////

//----------------------------------------------------------------------
// RXgN^
//----------------------------------------------------------------------

MyMailAddressInfo::MyMailAddressInfo()
{
	ZeroMemory(&m_lastWTimeFolderList, sizeof(m_lastWTimeFolderList));
}

//----------------------------------------------------------------------
// Becky! ̃f[^tH_ݒ肷
//----------------------------------------------------------------------

void MyMailAddressInfo::SetBkDataFolder(char* bkaDataFolder)
{
	m_bkaDataFolder = bkaDataFolder;
	if (!Util::EndsWithPathDelimiter(m_bkaDataFolder)) {
		m_bkaDataFolder += "\\";
	}

	m_folderListFilepath = m_bkaDataFolder;
	m_folderListFilepath += "Folder.lst";
}

//----------------------------------------------------------------------
// [AhX̏XV
//----------------------------------------------------------------------

void MyMailAddressInfo::Update()
{
#define __FUNCNAME__ "MyMailAddressInfo::Update()"

	WIN32_FILE_ATTRIBUTE_DATA fAttr;

	BOOL rc = GetFileAttributesEx(m_folderListFilepath.c_str(), GetFileExInfoStandard, &fAttr);
	if (!rc) {
		return;
	}

	if (fAttr.ftLastWriteTime.dwHighDateTime != m_lastWTimeFolderList.dwHighDateTime
	 || fAttr.ftLastWriteTime.dwLowDateTime != m_lastWTimeFolderList.dwLowDateTime) {

		// Folder.lst XVĂ
		m_lastWTimeFolderList = fAttr.ftLastWriteTime;

	} else {
		// ŏIXVԂςĂȂꍇ͉Ȃ
		return;
	}

	// [{bNX̏i[ݒt@C̃pXꗗ𓾂
	std::vector<std::string> mailBoxInfoFilepathList;

	GetMailBoxInfoFilepathList(mailBoxInfoFilepathList);

	// e[{bNX̐ݒt@C玩g̃[AhẌꗗ𓾂

	std::set<std::string> myEmailList;

	for (std::vector<std::string>::iterator it = mailBoxInfoFilepathList.begin(); it != mailBoxInfoFilepathList.end(); ++it) {
		char buffer[1024];
		buffer[0] = '\0';

		GetPrivateProfileString("Account", "MailAddress", "", buffer, sizeof(buffer), it->c_str());
		std::string email = Util::Trim(buffer);
		if (email.length() != 0) {
			myEmailList.insert(email);
		}
	}

	m_myEmailList = myEmailList;

	LOGGING_DEBUG("update m_myEmailList.");
	for (std::set<std::string>::iterator it = m_myEmailList.begin(); it != m_myEmailList.end(); ++it) {
		LOGGING_DEBUG("m_myEmailList: [%s].", it->c_str());
	}

#undef __FUNCNAME__
}

//----------------------------------------------------------------------
// [AhXg̃[AhẌꗗɑ݂邩𓾂
//----------------------------------------------------------------------

bool MyMailAddressInfo::IsExist(const char* email)
{
	std::set<std::string>::iterator it = m_myEmailList.find(email);
	if (it != m_myEmailList.end()) {
		return true;
	}

	return false;
}

//----------------------------------------------------------------------
// [AhX̃hCg̃[AhẌꗗɑ݂邩𓾂
//----------------------------------------------------------------------

bool MyMailAddressInfo::IsDomainExist(const char* emailDomain)
{
	for (std::set<std::string>::iterator it = m_myEmailList.begin(); it != m_myEmailList.end(); ++it) {
		std::string myDomain = Util::GetAtNext(it->c_str());
		if (myDomain.length() == 0) {
			continue;
		}

		if (strcmp(emailDomain, myDomain.c_str()) == 0) {
			return true;
		}
	}

	return false;
}

//----------------------------------------------------------------------
// [{bNX̏i[ݒt@C̃pXꗗ𓾂
//----------------------------------------------------------------------

void MyMailAddressInfo::GetMailBoxInfoFilepathList(std::vector<std::string>& mailBoxInfoFilepathList)
{
	std::ifstream folderListFile;
	folderListFile.open(m_folderListFilepath.c_str(), std::ios::in);
	if (folderListFile) {
		std::string line;
		while (std::getline(folderListFile, line)) {

			std::string mailboxId = Util::Trim(line);
			if (mailboxId.length() == 0) {
				continue;
			}

			char infoFilepath[MAX_PATH * 4];
			_snprintf(infoFilepath, sizeof(infoFilepath), "%s%s\\Mailbox.ini", m_bkaDataFolder.c_str(), mailboxId.c_str());
			FORCE_ASCIZ(infoFilepath);

			mailBoxInfoFilepathList.push_back(infoFilepath);
		}

		folderListFile.close();
	}
}

